import os, logging, platform


def log_init():
	'''加入run.log，信息打印到屏幕和log文件'''
	logging.basicConfig(level=logging.INFO,
				format='%(asctime)s %(filename)s[line:%(lineno)d] %(levelname)s %(message)s',
				datefmt='%a, %d %b %Y %H:%M:%S',
				filename='run.log',
				filemode='w')
	console = logging.StreamHandler()
	console.setLevel(logging.INFO)
	formatter = logging.Formatter('%(name)-12s: %(levelname)-8s %(message)s')
	console.setFormatter(formatter)
	logging.getLogger('').addHandler(console)

	if platform.system() == 'Windows':
		logging.info('执行系统：Windows')
	elif platform.system() == 'Linux':
		logging.info('执行系统：Linux')
	else:
		logging.info('执行系统：Unknown!')




import os, sys


path = '/home/gift/Desktop/新建文件夹'
namelist = os.listdir(path)

for item in namelist:
	if 'GB-T' in item:
		oldname = item
		newname = item.replace('GB-T', 'GB')
		os.rename(os.sep.join([path, oldname]), os.sep.join([path, newname]))




from selenium import webdriver
from selenium.webdriver.support.wait import WebDriverWait
from selenium.webdriver.support import expected_conditions as EC
from selenium.webdriver.common.by import By
from email.mime.multipart import MIMEMultipart    
from email.mime.text import MIMEText    
from email.mime.image import MIMEImage 
from email.header import Header   
from datetime import datetime
from time import sleep
import smtplib, platform, os, logging, traceback


class autoom():


	def __init__(self, crontab_time, delay_start=False):

		self.workdir 			= os.getcwd()
		self.crontab_time 		= crontab_time # everyday exec time
		self.delay_start 		= delay_start # False means start process this time, True means next time
		self.receivers 			= [] #['xx@xxx.com']
		self.smtpserver 		= '' #'smtp.exmail.qq.com'
		self.mail_username 		= ''# 'xx@xxx.com'
		self.mail_password		= ''# sender email password
		self.mail_sender		= ''# sender email 'xx@xx.com'
		self.reportfile         = "%s"%(self.workdir)+"report.txt" # report.txt

	def sleep_second(self, crontab_time, now):
		'''crontab sleep time'''
		if (crontab_time-now).total_seconds() >= 0:
			return float((crontab_time-now).total_seconds())
		elif (crontab_time-now).total_seconds() < 0:
			return float(86400.00+(crontab_time-now).total_seconds())


	def mail(self, receiver):
		'''auto send email to receiver'''

		smtpserver = self.smtpserver
		msg = MIMEMultipart('mixed') 
		msg['Subject'] = self.subject
		msg['From'] = self.mail_sender
		msg['To'] = receiver
		with open(self.reportfile, 'r') as f:	# with open(reportfile, 'r', encoding='utf-8') / with open(reportfile, 'r', encoding='gbk')
			text = f.read()
		text_plain = MIMEText(text,'plain', 'utf-8')
		msg.attach(text_plain)    
		# smtp = smtplib.SMTP()
		smtp = smtplib.SMTP_SSL()
		smtp.connect('smtp.exmail.qq.com')
		smtp.login(self.mail_username, self.mail_password)    
		smtp.sendmail(self.mail_sender, receiver, msg.as_string())    
		smtp.quit()	


	def mail_content(receiver, shopind):
		pass


	def pycrontab(self):
		switch = "on"
		while True:
			now = datetime.strptime(str(datetime.now().year)+"-"+str(datetime.now().month)+"-"+str(datetime.now().day)+" "+str(datetime.now().hour)+":"+str(datetime.now().minute)+":"+str(datetime.now().second), "%Y-%m-%d %H:%M:%S")
			crontab_time = datetime.strptime(str(datetime.now().year)+"-"+str(datetime.now().month)+"-"+str(datetime.now().day)+" "+self.crontab_time, "%Y-%m-%d %H:%M:%S")
			if (now >= crontab_time) and (switch == "on"):
				switch = "off"
				try:
					with open(self.reportfile, 'w') as f:
						now = datetime.strptime(str(datetime.now().year)+"-"+str(datetime.now().month)+"-"+str(datetime.now().day)+" "+str(datetime.now().hour)+":"+str(datetime.now().minute)+":"+str(datetime.now().second), "%Y-%m-%d %H:%M:%S")
						f.write( "执行系统: "+str(platform.platform()))
						f.write("\n")
						f.write( "报告时间: "+str(now))
						f.write("\n")
						f.write("\n")
						
					self.main()

				except:
					logging.info("Exception: "+traceback.format_exc()) # print exception message
					with open(self.reportfile, 'a') as f:
						f.write("执行错误, 请检查!")
						f.write("Exception: "+traceback.format_exc())
						f.write('\n')
						f.write('\n')	

				finally:
					for receiver in self.receivers:
						self.mail(receiver)
			else:
				switch = "on"
				print(now)
				print("Waitting for the right time to start,%s s left..."%(str(self.sleep_second(crontab_time, now))))
				sleep(self.sleep_second(crontab_time, now)+10)


	def main(self):
		pass




from time import sleep
import os, threading, pyautogui, logging, traceback


class autogui():
	'''auto gui everything!'''

	def __init__(self):
		self.getter = None # get the ctrl+c content
		pyautogui.PAUSE = 0.5
		pyautogui.FAILSAFE = False


	def auto_login(self, url, username, passwd, pause=False):
		'''login in template decerator'''
		def wrapper(func):
			os.system('nohup google-chrome 2>&1 &') # 不占终端运行
			sleep(5)
			pyautogui.hotkey('ctrlleft', 'shiftleft', 'n')
			sleep(5)
			pyautogui.hotkey('altleft', 'tab')  # 关闭第一个正常窗口，否则窗口切换会混乱！
			sleep(5)
			pyautogui.hotkey('altleft', 'f4')
			sleep(5)
			# pyautogui.hotkey('ctrlleft', 'winleft', 'up')
			# pyautogui.hotkey('ctrlleft', 'altleft', '5') # Ubuntu server?  # 通过事先手动打开一个最大化的chrome解决！
			sleep(5)
			pyautogui.typewrite(url ,0.1)
			sleep(5)
			pyautogui.press('enter')
			pyautogui.click(500, 100, button='left')
			sleep(5)

			func()

			pyautogui.typewrite(username,0.1)
			sleep(5)
			pyautogui.press('tab')
			pyautogui.typewrite(passwd,0.1)
			sleep(5)
			if pause:
				pyautogui.alert('Next') # 手动输入验证码时，暂停
			pyautogui.press('enter')
			sleep(5)
			
		return wrapper


	def copy(self):
		'''use gedit to get the contnet'''
		os.system('nohup gedit temp 2>&1 &') # self.getter = pyautogui.prompt(text='', title='' , default='')


	def confirm(self):
		'''confirm the promopt frame'''
		pyautogui.hotkey('ctrlleft', 'v')
		sleep(3)
		pyautogui.hotkey('ctrlleft', 's')
		sleep(3)
		pyautogui.hotkey('altleft', 'f4')


	def get_copy(self):
		'''use threading to skip the block of the thread'''
		try:
			os.remove('temp')
			sleep(3)
		except:
			logging.info("Exception: "+traceback.format_exc()) # print exception message
			sleep(3)
		
		process_copy = threading.Thread(target=self.copy, name='process_copy')
		# sleep(2)
		sleep(3)
		pyautogui.hotkey('ctrlleft', 'c')
		# sleep(2)
		sleep(3)
		process_copy.start()
		# sleep(1)
		sleep(3)
		self.confirm()
		# sleep(2)
		sleep(3)

		with open ('temp', 'r') as temp:
			self.getter = temp.read()


	def get_position(self):
		'''get the position of mouse, should run this method in a terminal to quit easily'''
		print('Press Ctrl-C to quit!')
		try:
			while True:
				x, y = pyautogui.position()
				positionStr = 'X: {} Y: {}'.format(*[str(x).rjust(4) for x in [x, y]])
				print(positionStr, end='')
				print('\b' * len(positionStr), end='', flush=True)
		except KeyboardInterrupt:
			logging.info("Exception: "+traceback.format_exc()) # print exception message
			print('\n')




# ag = autogui()
# ag.get_copy()
# # ag.get_position()
# print(ag.getter)


from selenium import webdriver
from selenium.webdriver.support.wait import WebDriverWait
from selenium.webdriver.support import expected_conditions as EC
from selenium.webdriver.common.by import By
from datetime import datetime
from time import sleep
import time, os, random, platform, collections, csv, shutil, pyautogui, logging, traceback


class crawler():


	def __init__(self, url="", username="", passwd="", submit="", file_field=""):
		"""初始化，传递网址，账号密码，表头，使用驱动，结果文件，并使用logger日志等"""
		self.url = url
		self.username = username
		self.passwd = passwd
		self.submit = submit
		self.file_field = file_field
		# self.browser = webdriver.Firefox() # firefox打开主页显示不正常，舍弃...
		self.browser = None #webdriver.Chrome()  or webdriver.Firefox() 福建用firefox打开主页显示不正常，而国标委的用firefox更快！要按实际情况选择
		self.total_page = 0 # 目录总页数
		self.xp_nextpage = "" # 下一页的xp
		self.xp_prepage = "" # 上一页的xp
		self.list_total_num = 0 # 每页中的条目数量
		self.list_total_num_last = 0 # 最后一页中的条目数量
		self.counter = 0 # 下一页计数器
		self.csv_file = 'result.csv'
		self.workdir = os.getcwd()
		self.get_dict = collections.defaultdict()
		self.get_list = []
		self.start_page = 1 # 开始爬取的页码，默认从第一页开始爬，也可手动设定，用于断点续爬
		self.start_time = time.perf_counter() # 开始运行时间
		self.running_time = 0 # 已消耗时间，s
		self.total_time = 0 # 估计总共需要消耗时间，s
		self.remained_time = 0 # 估计剩余时间，s


	def exe_loop(self, command, browser, *para, maxcount=3):
		"""循环执行命令command，默认最多循环3次"""
		browser = browser
		if para:
			para = para[0]
			logging.info("Processing: "+command+", "+para)
		else:
			logging.info("Processing: "+command)
		counter = 0
		while True:
			try:
				eval(command)
				logging.info("Exe Success!")
				break
			except:
				logging.info('Exception: '+traceback.format_exc()) # print exception message
				sleep(random.uniform(1, 3))
				if counter >= maxcount:
					break
				else:
					logging.info("waitting for..."+str(counter)+"/"+str(maxcount)+", "+str(int(counter/maxcount*100))+"%")
					counter += 1


	def element_click(self, browser, element_list, sleep_time=5):
		for item in element_list:
			self.exe_loop("WebDriverWait(browser, 1).until(EC.presence_of_element_located((By.XPATH, para))).click()", browser, item)
			sleep(random.uniform(3, sleep_time))


	def get_reset(self):
		"""清空数据获取字典和列表"""
		self.get_dict = collections.defaultdict()
		self.get_list = []


	def get_runningtime(self, process):
		"""查看完成进度，消耗时间，总消耗时间和剩余时间，打印到log"""
		self.running_time = (time.perf_counter() - self.start_time) # 已消耗时间，s
		self.total_time = self.running_time/process # 估计总共需要消耗时间，s
		self.remained_time = self.total_time - self.running_time # 估计剩余时间，s

		logging.info("完成进度: "+str(round(process*100, 2))+"%")
		logging.info("已消耗时间: "+str(round(self.running_time, 2))+"s，"+str(round(self.running_time/3600, 2))+"h")
		logging.info("估计剩余时间: "+str(round(self.remained_time, 2))+"s，"+str(round(self.remained_time/3600, 2))+"h")
		logging.info("估计总共需要消耗时间: "+str(round(self.total_time, 2))+"s，"+str(round(self.total_time/3600, 2))+"h")


	def login(self):
		"""登录主页"""
		self.exe_loop("browser.get(para)", self.browser, self.url)
		self.browser.maximize_window()
		sleep(random.uniform(1, 3))


	def auto_refresh(self, fun_try, fun_except, *args, maxrefresh=3):
		"""首先判断该记录的获取页面是否成功，
			成功 -> 跳出循环
			失败 -> 刷新页面
				maxrefresh==-1时 -> 无限刷新到成功为止
				maxrefresh!=-1时 and 刷新次数>=maxrefresh 	-> 跳出while循环，继续后续程序
		"""
		
		counter_refresh = 0
		
		while True:
			try:
				fun_try(*args)
				break # 成功获取则退出
			except:
				logging.info('@auto_refresh: fun_try() exception!')
				logging.info('Exception: '+traceback.format_exc()) # print exception message
				sleep(random.uniform(1,3))

				try:
					pyautogui.press('f5') #	self.browser.refresh()会卡浏览器!
					sleep(random.uniform(1, 3))
					logging.info('@auto_refresh: browse has refreshed!') # print exception message
					get_fun_except = fun_except(*args)
					logging.info('@auto_refresh: fun_except() done!') # print exception message
					counter_refresh += 1
				except:
					logging.info('@auto_refresh: browse refreshed or fun_except() exception!')
					logging.info('Exception: '+traceback.format_exc()) # print exception message

				if (maxrefresh != -1) and counter_refresh >= maxrefresh: # 如果大于刷新次数的话，会返回一个字符串并跳出循环
					logging.info('@auto_refresh: skip refreshed and continue to next process!')
					return get_fun_except
					break # 获取不成功但超过最大刷新次数获取也退出


	def csv_export(self, data):
		"""创建并写入结果result.csv文件，首先判读result.csv是否存在，不存在则创建并写入表头，存在则增量写入文件。注意，不同操作系统的encoding是不同的。data格式：[{}]， self.file_field格式:['','']"""
		# 如果'result.csv'不存在，则创建文件，写入表头
		if not(os.path.exists(self.csv_file)):	
			logging.info('创建result.csv文件')
			# with open(csvfile_name, 'a', encoding='GBK', newline='') as csvfile: #FineBI
			if platform.system() == 'Windows':
				csvfile = open(self.csv_file, 'w', encoding='UTF8', newline='')
			elif platform.system() == 'Linux':
				csvfile = open(self.csv_file, 'w', newline='')
			writer = csv.DictWriter(csvfile, fieldnames=self.file_field)
			writer.writeheader()
			csvfile.close()
		
		# 如果'result.csv'存在，则增量写入文件
		logging.info('写入result.csv文件')
		if platform.system() == 'Windows':
			csvfile = open(self.csv_file, 'a', encoding='UTF8', newline='')
		elif platform.system() == 'Linux':
			csvfile = open(self.csv_file, 'a', newline='')
		writer = csv.DictWriter(csvfile, fieldnames=self.file_field)
		writer.writerows(data)
		csvfile.close()


	def done(self):
		"""成功完成后，copy文件，打印完成信息"""
		timenow = datetime.strptime(str(datetime.now().year)+"_"+str(datetime.now().month)+"_"+str(datetime.now().day)+"_"+str(datetime.now().hour)+":"+str(datetime.now().minute)+":"+str(datetime.now().second), "%Y_%m_%d_%H:%M:%S")

		try:
			os.mkdir("result")
		except:
			shutil.copyfile("result.csv", self.workdir+"/result/result_"+str(timenow).replace(' ', '_').replace(':', '_')+".csv")

		logging.info('------------------------------------------------------')
		logging.info('运行完毕，请检查并备份result.csv文件！')


	def page_list_num(self):
		"""调整每页条目数量"""
		pass


	def get_xp_std(self, index):
		"""获取标准表头的xpath"""
		pass


	def get_data(self):
		"""爬取模板"""
		pass



import pip
from subprocess import call
from pip._internal.utils.misc import get_installed_distributions


for dist in get_installed_distributions():
	"""Auto upgrade all the pip outdate package. """
    call("pip install --upgrade " + dist.project_name, shell=True)



#!/usr/bin/env python
#-*- coding:utf-8 -*-

#############################################
# File Name: setup.py
# Author: JunWang
# Mail: shutupwj@163.com
# Created Time:  2018-6-8 11:30:34
#############################################

from setuptools import setup, find_packages

setup(
    name = "Jskit",
    version = "0.2.4",
    keywords = ("pip", "tool kit"),
    description = "A personal tool kit",
    long_description = "A personal tool kit",
    license = "Apache License Version 2.0",

    url = "https://github.com/Jakkwj/Jskit",
    author = "JunWang",
    author_email = "shutupwj@163.com",

    packages = find_packages(),
    # packages = find_packages(''),
    # package_dir = {'':''},   
    # package_data = {'': ['*.xlsx']},
    include_package_data = True,
    exclude_package_data = {'': ['.gitignore'], '': ['*.m']}, 
    platforms = "any",
    install_requires = ["selenium", "pyautogui"]
)
